@@ -1,4 +1,4 @@
-/*      $Id: irrecord.c,v 5.75 2008/09/21 10:36:51 lirc Exp $      */
+/*      $Id: irrecord.c,v 5.95 2009/08/29 07:47:31 lirc Exp $      */
 
 /****************************************************************************
  ** irrecord.c **************************************************************
@@ -52,6 +52,7 @@
 #include "ir_remote.h"
 #include "config_file.h"
 #include "receive.h"
+#include "input_map.h"
 
 void flushhw(void);
 int resethw(void);
@@ -80,7 +81,7 @@ void get_scheme(struct ir_remote *remote, int interactive);
 struct lengths *get_max_length(struct lengths *first,unsigned int *sump);
 void unlink_length(struct lengths **first,struct lengths *remove);
 int get_trail_length(struct ir_remote *remote, int interactive);
-int get_lead_length(struct ir_remote *remote);
+int get_lead_length(struct ir_remote *remote, int interactive);
 int get_repeat_length(struct ir_remote *remote, int interactive);
 int get_header_length(struct ir_remote *remote, int interactive);
 int get_data_length(struct ir_remote *remote, int interactive);
@@ -96,7 +97,7 @@ const char *usage="Usage: %s [options] file\n";
 struct ir_remote remote;
 struct ir_ncode ncode;
 
-#define IRRECORD_VERSION "0.5"
+#define IRRECORD_VERSION "$Revision: 5.95 $"
 #define BUTTON 80+1
 #define RETRIES 10
 
@@ -120,7 +121,7 @@ lirc_t aeps = 100;
 #define TH_REPEAT      90
 #define TH_TRAIL       90
 #define TH_LEAD        90
-#define TH_IS_BIT      15
+#define TH_IS_BIT      10
 #define TH_RC6_SIGNAL 550
 
 #define MIN_GAP  20000
@@ -159,6 +160,7 @@ lirc_t emulation_readdata(lirc_t timeout)
 		{
 			current_code = emulation_data->codes;
 		}
+		current_rep = 0;
 		sum = 0;
 	}
 	else
@@ -189,6 +191,7 @@ lirc_t emulation_readdata(lirc_t timeout)
 			if(current_code->name == NULL)
 			{
 				current_code = NULL;
+				return emulation_readdata(timeout);
 			}
 			if(data == 0)
 			{
@@ -241,7 +244,7 @@ struct hardware hw_emulation=
 	"emulation"
 };
 
-static int iprintf(int interactive, char *format_str, ...)
+static int i_printf(int interactive, char *format_str, ...)
 {
 	va_list ap;
 	int ret = 0;
@@ -314,6 +317,7 @@ int main(int argc,char **argv)
 	int repeat_flag;
 	lirc_t min_remaining_gap, max_remaining_gap;
 	int force;
+	int disable_namespace = 0;
 	int retries;
 	int no_data = 0;
 	struct ir_remote *remotes=NULL;
@@ -338,6 +342,8 @@ int main(int argc,char **argv)
 			{"device",required_argument,NULL,'d'},
 			{"driver",required_argument,NULL,'H'},
 			{"force",no_argument,NULL,'f'},
+			{"disable-namespace",no_argument,NULL,'n'},
+			{"list-namespace",no_argument,NULL,'l'},
 #ifdef DEBUG
 			{"pre",no_argument,NULL,'p'},
 			{"post",no_argument,NULL,'P'},
@@ -348,9 +354,9 @@ int main(int argc,char **argv)
 			{0, 0, 0, 0}
 		};
 #ifdef DEBUG
-		c = getopt_long(argc,argv,"hvad:H:fpPtiT",long_options,NULL);
+		c = getopt_long(argc,argv,"hvad:H:fnlpPtiT",long_options,NULL);
 #else
-		c = getopt_long(argc,argv,"hvad:H:f",long_options,NULL);
+		c = getopt_long(argc,argv,"hvad:H:fnl",long_options,NULL);
 #endif
 		if(c==-1)
 			break;
@@ -362,6 +368,8 @@ int main(int argc,char **argv)
 			printf("\t -v --version\t\tdisplay version\n");
 			printf("\t -a --analyse\t\tanalyse raw_codes config files\n");
 			printf("\t -f --force\t\tforce raw mode\n");
+			printf("\t -n --disable-namespace\t\tdisables namespace checks\n");
+			printf("\t -l --list-namespace\t\tlist valid button names\n");
 			printf("\t -H --driver=driver\tuse given driver\n");
 			printf("\t -d --device=device\tread from given device\n");
 			exit(EXIT_SUCCESS);
@@ -385,6 +393,13 @@ int main(int argc,char **argv)
 		case 'f':
 			force=1;
 			break;
+		case 'n':
+			disable_namespace = 1;
+			break;
+		case 'l':
+			fprint_namespace(stdout);
+			exit(EXIT_SUCCESS);
+			break;
 #ifdef DEBUG
 		case 'p':
 			get_pre=1;
@@ -427,7 +442,6 @@ int main(int argc,char **argv)
 			progname);
 		exit(EXIT_FAILURE);
 	}
-	aeps = (hw.resolution>aeps ? hw.resolution:aeps);
 	filename=argv[optind];
 	fin=fopen(filename,"r");
 	if(fin!=NULL)
@@ -479,8 +493,6 @@ int main(int argc,char **argv)
 		remote.codes=NULL;
 		remote.last_code=NULL;
 		remote.next=NULL;
-		remote.toggle_bit=0;
-		remote.toggle_bit_mask=0;
 		if(remote.pre_p==0 && remote.pre_s==0 &&
 		   remote.post_p==0 && remote.post_s==0)
 		{
@@ -542,6 +554,7 @@ int main(int argc,char **argv)
 			exit(EXIT_FAILURE);
 		}
 	}
+	aeps = (hw.resolution>aeps ? hw.resolution:aeps);
 	
 	if(hw.rec_mode==LIRC_MODE_STRING)
 	{
@@ -576,7 +589,17 @@ int main(int argc,char **argv)
 	printf(
 "This program will record the signals from your remote control\n"
 "and create a config file for lircd.\n\n"
-"\n"
+"\n");
+	if(hw.name && strcmp(hw.name, "devinput") == 0)
+	{
+		printf(
+"Usually it's not necessary to create a new config file for devinput\n"
+"devices. A generic config file can be found at:\n"
+"http://www.lirc.org/remotes/devinput/\n"
+"You should try this config file before creating your own config file.\n"
+"\n");
+	}
+	printf(
 "A proper config file for lircd is maybe the most vital part of this\n"
 "package, so you should invest some time to create a working config\n"
 "file. Although I put a good deal of effort in this program it is often\n"
@@ -608,7 +631,7 @@ int main(int argc,char **argv)
 	switch(hw.rec_mode)
 	{
 	case LIRC_MODE_MODE2:
-		if(!using_template && !get_lengths(&remote, force, 0))
+		if(!using_template && !get_lengths(&remote, force, 1))
 		{
 			if(remote.gap==0)
 			{
@@ -717,6 +740,17 @@ int main(int argc,char **argv)
 		{
 			break;
 		}
+		if(!disable_namespace && !is_in_namespace(buffer))
+		{
+			printf("'%s' is not in name space (use "
+			       "--disable-namespace to disable checks)\n",
+			       buffer);
+			printf("Use '%s --list-namespace' to see a full list "
+			       "of valid button names\n",
+			       progname);
+			printf("Please try again.\n");
+			continue;
+		}
 		
 		if(is_raw(&remote))
 		{
@@ -922,7 +956,9 @@ int main(int argc,char **argv)
 	
 	if(!has_toggle_bit_mask(remotes))
 	{
-		if(!using_template) get_toggle_bit_mask(remotes);
+		if(!using_template &&
+		   strcmp(hw.name, "devinput") != 0)
+			get_toggle_bit_mask(remotes);
 	}
 	else
 	{
@@ -1376,10 +1412,11 @@ void for_each_remote(struct ir_remote *remotes, remote_func func)
 void analyse_remote(struct ir_remote *raw_data)
 {
 	struct ir_ncode *codes;
-	ir_code pre, code, post;
+	ir_code pre, code, code2, post;
 	int repeat_flag;
 	lirc_t min_remaining_gap, max_remaining_gap;
-	struct ir_ncode new_codes[100];
+	struct ir_ncode *new_codes;
+	size_t new_codes_count = 100;
 	int new_index = 0;
 	int ret;
 
@@ -1395,18 +1432,27 @@ void analyse_remote(struct ir_remote *raw_data)
 	next_code = NULL;
 	current_code = NULL;
 	current_index = 0;
+	memset(&remote, 0, sizeof(remote));
 	get_lengths(&remote, 0, 0 /* not interactive */ );
 
-	if(is_rc6(&remote))
+	if(is_rc6(&remote) && remote.bits >= 5)
 	{
 		/* have to assume something as it's very difficult to
 		   extract the rc6_mask from the data that we have */
-		remote.rc6_mask = (ir_code) 0x100000000ll;
+		remote.rc6_mask = ((ir_code) 0x1ll) << (remote.bits-5);
 	}
 
 	remote.name = raw_data->name;
+	remote.freq = raw_data->freq;
 	
-	memset(new_codes, 0, sizeof(new_codes));
+	new_codes = malloc(new_codes_count * sizeof(*new_codes));
+	if(new_codes == NULL)
+	{
+		fprintf(stderr, "%s: out of memory\n",
+			progname);
+		return;
+	}
+	memset(new_codes, 0 , new_codes_count * sizeof(*new_codes));
 	codes = raw_data->codes;
 	while(codes->name!=NULL)
 	{
@@ -1429,8 +1475,42 @@ void analyse_remote(struct ir_remote *raw_data)
 		}
 		else
 		{
+			if(new_index+1 >= new_codes_count)
+			{
+				struct ir_ncode *renew_codes;
+				
+				new_codes_count *= 2;
+				renew_codes = realloc
+					(new_codes,
+					 new_codes_count * sizeof(*new_codes));
+				if(renew_codes == NULL)
+				{
+					fprintf(stderr, "%s: out of memory\n",
+						progname);
+					free(new_codes);
+					return;
+				}
+				memset(&new_codes[new_codes_count/2], 0 , new_codes_count/2 * sizeof(*new_codes));
+				new_codes = renew_codes;
+			}
+			
+			clear_rec_buffer();
+			ret = receive_decode(&remote,
+					     &pre, &code2, &post,
+					     &repeat_flag,
+					     &min_remaining_gap,
+					     &max_remaining_gap);
+			if(ret && code2 != code)
+			{
+				new_codes[new_index].next = malloc(sizeof(*(new_codes[new_index].next)));
+				if(new_codes[new_index].next)
+				{
+					memset(new_codes[new_index].next, 0, sizeof(*(new_codes[new_index].next)));
+					new_codes[new_index].next->code = code2;
+				}
+			}
 			new_codes[new_index].name = codes->name;
-			new_codes[new_index].code = code;
+			new_codes[new_index].code = code;			
 			new_index++;
 		}
 		codes++;
@@ -1438,6 +1518,8 @@ void analyse_remote(struct ir_remote *raw_data)
 	new_codes[new_index].name = NULL;
 	remote.codes = new_codes;
 	fprint_remotes(stdout, &remote);
+	remote.codes = NULL;
+	free(new_codes);
 }
 
 #ifdef DEBUG
@@ -1610,7 +1692,7 @@ inline lirc_t calc_signal(struct lengths *len)
 int get_lengths(struct ir_remote *remote, int force, int interactive)
 {
 	int retval;
-	lirc_t data,average,sum,remaining_gap,header;
+	lirc_t data,average,maxspace,sum,remaining_gap,header;
 	enum analyse_mode mode=MODE_GAP;
 	int first_signal;
 
@@ -1628,7 +1710,7 @@ int get_lengths(struct ir_remote *remote, int force, int interactive)
 		flushhw();
 	}
 	retval=1;
-	average=0;sum=0;count=0;count_spaces=0;
+	average=0;maxspace=0;sum=0;count=0;count_spaces=0;
 	count_3repeats=0;count_5repeats=0;count_signals=0;
 	first_signal=-1;header=0;
 	first_length=0;
@@ -1657,12 +1739,15 @@ int get_lengths(struct ir_remote *remote, int force, int interactive)
 					continue;
 				}
 				average=data;
+				maxspace=data;
 			}
 			else if(is_space(data))
 			{
 				if(data>MIN_GAP || data>100*average ||
 				   /* this MUST be a gap */
-				   (count_spaces>10 && data>5*average))  
+				   (count_spaces>10 && data>5*maxspace/2)
+				   /* || Echostar
+				      (count_spaces>20 && data>9*maxspace/10)*/)
 					/* this should be a gap */
 				{
 					struct lengths *scan;
@@ -1674,7 +1759,7 @@ int get_lengths(struct ir_remote *remote, int force, int interactive)
 					merge_lengths(first_sum);
 					add_length(&first_gap,data);
 					merge_lengths(first_gap);
-					sum=0;count_spaces=0;average=0;
+					sum=0;count_spaces=0;average=0;maxspace=0;
 					
 					maxcount=0;
 					scan=first_sum;
@@ -1686,7 +1771,7 @@ int get_lengths(struct ir_remote *remote, int force, int interactive)
 						{
 							remote->gap=calc_signal(scan);
 							remote->flags|=CONST_LENGTH;
-							iprintf(interactive, "\nFound const length: %lu\n",(unsigned long) remote->gap);
+							i_printf(interactive, "\nFound const length: %lu\n",(unsigned long) remote->gap);
 							break;
 						}
 						scan=scan->next;
@@ -1701,7 +1786,7 @@ int get_lengths(struct ir_remote *remote, int force, int interactive)
 							if(scan->count>SAMPLES)
 							{
 								remote->gap=calc_signal(scan);
-								iprintf(interactive, "\nFound gap: %lu\n",(unsigned long) remote->gap);
+								i_printf(interactive, "\nFound gap: %lu\n",(unsigned long) remote->gap);
 								break;
 							}
 							scan=scan->next;
@@ -1709,7 +1794,7 @@ int get_lengths(struct ir_remote *remote, int force, int interactive)
 					}
 					if(scan!=NULL)
 					{
-						iprintf(interactive, "Please keep on pressing buttons like described above.\n");
+						i_printf(interactive, "Please keep on pressing buttons like described above.\n");
 						mode=MODE_HAVE_GAP;
 						sum=0;
 						count=0;
@@ -1740,6 +1825,10 @@ int get_lengths(struct ir_remote *remote, int force, int interactive)
 				average=(average*count_spaces+data)
 				/(count_spaces+1);
 				count_spaces++;
+				if(data>maxspace)
+				{
+					maxspace=data;
+				}
 			}
 			if(count>SAMPLES*MAX_SIGNALS*2)
 			{
@@ -1773,8 +1862,9 @@ int get_lengths(struct ir_remote *remote, int force, int interactive)
 			}
 			sum+=data&PULSE_MASK;
 
-			if((data&PULSE_MASK)>=remaining_gap*(100-eps)/100
-			   || (data&PULSE_MASK)>=remaining_gap-aeps)
+			if(count>2 &&
+			   ((data&PULSE_MASK)>=remaining_gap*(100-eps)/100
+			    || (data&PULSE_MASK)>=remaining_gap-aeps))
 			{
 				if(is_space(data))
 				{
@@ -1878,11 +1968,11 @@ int get_lengths(struct ir_remote *remote, int force, int interactive)
 #endif
 				if(count_signals>=SAMPLES)
 				{
-					iprintf(interactive, "\n");
+					i_printf(interactive, "\n");
 					get_scheme(remote, interactive);
 					if(!get_header_length(remote, interactive) ||
 					   !get_trail_length(remote, interactive) ||
-					   !get_lead_length(remote) ||
+					   !get_lead_length(remote, interactive) ||
 					   !get_repeat_length(remote, interactive) ||
 					   !get_data_length(remote, interactive))
 					{
@@ -2061,8 +2151,8 @@ void get_scheme(struct ir_remote *remote, int interactive)
 	if(lengths[length]>=TH_SPACE_ENC*sum/100)
 	{
 		length++;
-		iprintf(interactive, "Space/pulse encoded remote control found.\n");
-		iprintf(interactive, "Signal length is %u.\n",length);
+		i_printf(interactive, "Space/pulse encoded remote control found.\n");
+		i_printf(interactive, "Signal length is %u.\n",length);
 		/* this is not yet the
 		   number of bits */
 		remote->bits=length;
@@ -2099,17 +2189,18 @@ void get_scheme(struct ir_remote *remote, int interactive)
 				
 				maxs=get_max_length(first_space,NULL);
 				
-				if((calc_signal(maxp)<TH_RC6_SIGNAL ||
+				if(length > 20 &&
+				   (calc_signal(maxp)<TH_RC6_SIGNAL ||
 				    calc_signal(max2p)<TH_RC6_SIGNAL) &&
 				   (calc_signal(maxs)<TH_RC6_SIGNAL ||
 				    calc_signal(max2s)<TH_RC6_SIGNAL))
 				{
-					iprintf(interactive, "RC-6 remote control found.\n");
+					i_printf(interactive, "RC-6 remote control found.\n");
 					set_protocol(remote, RC6);
 				}
 				else
 				{
-					iprintf(interactive, "RC-5 remote control found.\n");
+					i_printf(interactive, "RC-5 remote control found.\n");
 					set_protocol(remote, RC5);
 				}
 				return;
@@ -2117,7 +2208,7 @@ void get_scheme(struct ir_remote *remote, int interactive)
 		}
 	}
 	length++;
-	iprintf(interactive, "Suspicious data length: %u.\n",length);
+	i_printf(interactive, "Suspicious data length: %u.\n",length);
 	/* this is not yet the number of bits */
 	remote->bits=length;
 	set_protocol(remote, SPACE_ENC);
@@ -2132,6 +2223,10 @@ struct lengths *get_max_length(struct lengths *first,unsigned int *sump)
 	max_length=first;
 	sum=first->count;
 	
+#       ifdef DEBUG
+	if(first->count>0) printf("%u x %lu\n", first->count,
+				  (unsigned long) calc_signal(first));
+#       endif
 	scan=first->next;
 	while(scan)
 	{
@@ -2164,22 +2259,22 @@ int get_trail_length(struct ir_remote *remote, int interactive)
 #       endif
 	if(max_count>=sum*TH_TRAIL/100)
 	{
-		iprintf(interactive, "Found trail pulse: %lu\n",
+		i_printf(interactive, "Found trail pulse: %lu\n",
 			(unsigned long) calc_signal(max_length));
 		remote->ptrail=calc_signal(max_length);
 		return(1);
 	}
-	iprintf(interactive, "No trail pulse found.\n");
+	i_printf(interactive, "No trail pulse found.\n");
 	return(1);
 }
 
-int get_lead_length(struct ir_remote *remote)
+int get_lead_length(struct ir_remote *remote, int interactive)
 {
 	unsigned int sum,max_count;
 	struct lengths *first_lead,*max_length,*max2_length;
 	lirc_t a,b,swap;
 	
-	if(!is_biphase(remote)) return(1);
+	if(!is_biphase(remote) || has_header(remote)) return(1);
 	if(is_rc6(remote)) return(1);
 	
 	first_lead=has_header(remote) ? first_3lead:first_1lead;
@@ -2190,7 +2285,7 @@ int get_lead_length(struct ir_remote *remote)
 #       endif
 	if(max_count>=sum*TH_LEAD/100)
 	{
-		printf("Found lead pulse: %lu\n",
+		i_printf(interactive, "Found lead pulse: %lu\n",
 		       (unsigned long) calc_signal(max_length));
 		remote->plead=calc_signal(max_length);
 		return(1);
@@ -2207,12 +2302,12 @@ int get_lead_length(struct ir_remote *remote)
 	}
 	if(abs(2*a-b)<b*eps/100 || abs(2*a-b)<aeps)
 	{
-		printf("Found hidden lead pulse: %lu\n",
+		i_printf(interactive, "Found hidden lead pulse: %lu\n",
 		       (unsigned long) a);
 		remote->plead=a;
 		return(1);
 	}
-	printf("No lead pulse found.\n");
+	i_printf(interactive, "No lead pulse found.\n");
 	return(1);
 }
 
@@ -2229,7 +2324,7 @@ int get_header_length(struct ir_remote *remote, int interactive)
 	}
 	else
 	{
-		iprintf(interactive, "No header data.\n");
+		i_printf(interactive, "No header data.\n");
 		return(1);
 	}
 #       ifdef DEBUG
@@ -2249,20 +2344,20 @@ int get_header_length(struct ir_remote *remote, int interactive)
 			headerp=calc_signal(max_plength);
 			headers=calc_signal(max_slength);
 
-			iprintf(interactive, "Found possible header: %lu %lu\n",
+			i_printf(interactive, "Found possible header: %lu %lu\n",
 				(unsigned long) headerp,
 				(unsigned long) headers);
 			remote->phead=headerp;
 			remote->shead=headers;
 			if(first_lengths<second_lengths)
 			{
-				iprintf(interactive, "Header is not being repeated.\n");
+				i_printf(interactive, "Header is not being repeated.\n");
 				remote->flags|=NO_HEAD_REP;
 			}
 			return(1);
 		}
 	}
-	iprintf(interactive, "No header found.\n");
+	i_printf(interactive, "No header found.\n");
 	return(1);
 }
 
@@ -2280,7 +2375,7 @@ int get_repeat_length(struct ir_remote *remote, int interactive)
 			printf("Repeat inconsitentcy.\n");
 			return(0);
 		}
-		iprintf(interactive, "No repeat code found.\n");
+		i_printf(interactive, "No repeat code found.\n");
 		return(1);
 	}
 
@@ -2315,7 +2410,7 @@ int get_repeat_length(struct ir_remote *remote, int interactive)
 			repeatp=calc_signal(max_plength);
 			repeats=calc_signal(max_slength);
 			
-			iprintf(interactive, "Found repeat code: %lu %lu\n",
+			i_printf(interactive, "Found repeat code: %lu %lu\n",
 				(unsigned long) repeatp,
 				(unsigned long) repeats);
 			remote->prepeat=repeatp;
@@ -2325,7 +2420,7 @@ int get_repeat_length(struct ir_remote *remote, int interactive)
 				max_slength=get_max_length(first_repeat_gap,
 							   NULL);
 				repeat_gap=calc_signal(max_slength);
-				iprintf(interactive, "Found repeat gap: %lu\n",
+				i_printf(interactive, "Found repeat gap: %lu\n",
 					(unsigned long) repeat_gap);
 				remote->repeat_gap=repeat_gap;
 				
@@ -2333,7 +2428,7 @@ int get_repeat_length(struct ir_remote *remote, int interactive)
 			return(1);
 		}
 	}
-	iprintf(interactive, "No repeat header found.\n");
+	i_printf(interactive, "No repeat header found.\n");
 	return(1);
 
 }
@@ -2438,7 +2533,7 @@ int get_data_length(struct ir_remote *remote, int interactive)
 					printf("Unknown encoding found.\n");
 					return(0);
 				}
-				iprintf(interactive, "Signals are biphase encoded.\n");
+				i_printf(interactive, "Signals are biphase encoded.\n");
 				p1=calc_signal(max_plength);
 				p2=calc_signal(max2_plength);
 				s1=calc_signal(max_slength);
@@ -2467,7 +2562,7 @@ int get_data_length(struct ir_remote *remote, int interactive)
 				if(max2_plength)
 				{
 					p2=calc_signal(max2_plength);
-					iprintf(interactive, "Signals are pulse encoded.\n");
+					i_printf(interactive, "Signals are pulse encoded.\n");
 					remote->pone=max(p1,p2);
 					remote->sone=s1;
 					remote->pzero=min(p1,p2);
@@ -2481,7 +2576,7 @@ int get_data_length(struct ir_remote *remote, int interactive)
 				else
 				{
 					s2=calc_signal(max2_slength);
-					iprintf(interactive, "Signals are space encoded.\n");
+					i_printf(interactive, "Signals are space encoded.\n");
 					remote->pone=p1;
 					remote->sone=max(s1,s2);
 					remote->pzero=p1;
@@ -2500,7 +2595,7 @@ int get_data_length(struct ir_remote *remote, int interactive)
 				{
 					remote->phead=remote->shead=0;
 					remote->flags&=~NO_HEAD_REP;
-					iprintf(interactive, "Removed header.\n");
+					i_printf(interactive, "Removed header.\n");
 				}
 				if(is_biphase(remote) &&
 				   expect(remote,remote->shead,remote->sone))
@@ -2508,7 +2603,7 @@ int get_data_length(struct ir_remote *remote, int interactive)
 					remote->plead=remote->phead;
 					remote->phead=remote->shead=0;
 					remote->flags&=~NO_HEAD_REP;
-					iprintf(interactive, "Removed header.\n");
+					i_printf(interactive, "Removed header.\n");
 				}
 			}
 			if(is_biphase(remote))
@@ -2535,7 +2630,7 @@ int get_data_length(struct ir_remote *remote, int interactive)
 					      (has_header(remote) ? 2:0)+
 					      1-(remote->ptrail>0 ? 2:0))/2;
 			}
-			iprintf(interactive, "Signal length is %d\n",remote->bits);
+			i_printf(interactive, "Signal length is %d\n",remote->bits);
 			free_lengths(&max_plength);
 			free_lengths(&max_slength);
 			return(1);
@@ -2591,8 +2686,6 @@ int get_gap_length(struct ir_remote *remote)
 				if(scan->count>SAMPLES)
 				{
 					remote->gap=calc_signal(scan);
-					/* this does not work very reliably */
-					remote->gap+=100000;
 					printf("\nFound gap length: %lu\n",
 					       (unsigned long) remote->gap);
 					free_lengths(&gaps);
