@@ -1,4 +1,4 @@
-/*      $Id: lircd.c,v 5.84 2009/04/28 16:25:50 lirc Exp $      */
+/*      $Id: lircd.c,v 5.91 2009/08/29 07:46:44 lirc Exp $      */
 
 /****************************************************************************
  ** lircd.c *****************************************************************
@@ -64,7 +64,7 @@
 #endif
 
 #if defined __APPLE__
-#include <sys/ioccom.h>
+#include <sys/ioctl.h>
 #endif
 
 #ifndef timersub
@@ -96,11 +96,12 @@ extern struct ir_ncode *repeat_code;
 
 static int repeat_fd=-1;
 static char *repeat_message=NULL;
+static unsigned int repeat_max = REPEAT_MAX_DEFAULT;
 
 extern struct hardware hw;
 
 char *progname="lircd";
-char *configfile=LIRCDCFGFILE;
+const char *configfile = NULL;
 #ifndef USE_SYSLOG
 char *logfile=LOGFILE;
 #else
@@ -155,10 +156,9 @@ char hostname[HOSTNAME_LEN+1];
 FILE *lf=NULL;
 #endif
 
-/* quite arbitrary limits */
-#define MAX_PEERS	100
 /* substract one for lirc, sockfd, sockinet, logfile, pidfile, uinput */
-#define MAX_CLIENTS     (FD_SETSIZE-6-MAX_PEERS)
+#define MAX_PEERS	((FD_SETSIZE-6)/2)
+#define MAX_CLIENTS     ((FD_SETSIZE-6)/2)
 
 int sockfd, sockinet;
 static int uinputfd = -1;
@@ -328,12 +328,14 @@ void dosigterm(int sig)
 	shutdown(sockfd,2);
 	close(sockfd);
 
+#if defined(__linux__)
 	if(uinputfd != -1)
 	{
 		ioctl(uinputfd, UI_DEV_DESTROY);
 		close(uinputfd);
 		uinputfd = -1;
 	}
+#endif
 	if(listen_tcpip)
 	{
 		shutdown(sockinet,2);
@@ -469,6 +471,8 @@ void config(void)
 {
 	FILE *fd;
 	struct ir_remote *config_remotes;
+	const char *filename = configfile;
+	if(filename == NULL) filename = LIRCDCFGFILE;
 	
 	if(free_remotes!=NULL)
 	{
@@ -476,14 +480,29 @@ void config(void)
 		logprintf(LOG_ERR,"old config is still in use");
 		return;
 	}
-	fd=fopen(configfile,"r");
+	fd=fopen(filename, "r");
+	if(fd == NULL && errno == ENOENT && configfile == NULL)
+	{
+		/* try old lircd.conf location */
+		int save_errno = errno;
+		fd = fopen(LIRCDOLDCFGFILE, "r");
+		if(fd != NULL)
+		{
+			filename = LIRCDOLDCFGFILE;
+		}
+		else
+		{
+			errno = save_errno;
+		}
+	}
 	if(fd==NULL)
 	{
-		logprintf(LOG_ERR,"could not open config file '%s'",
-			  configfile);
-		logperror(LOG_ERR,NULL);
+		logprintf(LOG_ERR, "could not open config file '%s'",
+			  filename);
+		logperror(LOG_ERR, NULL);
 		return;
 	}
+	configfile = filename;
 	config_remotes=read_config(fd, configfile);
 	fclose(fd);
 	if(config_remotes==(void *) -1)
@@ -1114,16 +1133,17 @@ void dosigalrm(int sig)
 }
 
 int parse_rc(int fd,char *message,char *arguments,struct ir_remote **remote,
-	     struct ir_ncode **code,int *reps,int n)
+	     struct ir_ncode **code, int *reps, int n, int *err)
 {
 	char *name=NULL,*command=NULL,*repeats,*end_ptr=NULL;
 
 	*remote=NULL;
 	*code=NULL;
-	if(arguments==NULL) return(1);
+	*err = 1;
+	if(arguments==NULL) goto arg_check;
 
 	name=strtok(arguments,WHITE_SPACE);
-	if(name==NULL) return(1);
+	if(name == NULL) goto arg_check;
 	*remote=get_ir_remote(remotes,name);
 	if(*remote==NULL)
 	{
@@ -1131,11 +1151,11 @@ int parse_rc(int fd,char *message,char *arguments,struct ir_remote **remote,
 				  name));
 	}
 	command=strtok(NULL,WHITE_SPACE);
-	if(command==NULL) return(1);
+	if(command == NULL) goto arg_check;
 	*code=get_code_by_name(*remote,command);
 	if(*code==NULL)
 	{
-		return(send_error(fd,message,"unknown command: \"%s\"\n",
+		return(send_error(fd,message, "unknown command: \"%s\"\n",
 				  command));
 	}
 	if(reps!=NULL)
@@ -1149,11 +1169,12 @@ int parse_rc(int fd,char *message,char *arguments,struct ir_remote **remote,
 				return(send_error(fd,message,
 						  "bad send packet\n"));
 			}
-			if (*reps>REPEAT_MAX)
+			if (*reps > repeat_max)
 			{
 				return(send_error
 				       (fd,message,
-					"too many repeats: \"%d\"\n",*reps));
+					"too many repeats: \"%d\" > \"%u\"\n",
+					*reps, repeat_max));
 			}
 		}
 		else
@@ -1165,6 +1186,7 @@ int parse_rc(int fd,char *message,char *arguments,struct ir_remote **remote,
 	{
 		return(send_error(fd,message,"bad send packet\n"));
 	}
+ arg_check:
 	if(n>0 && *remote==NULL)
 	{
 		return(send_error(fd,message,"remote missing\n"));
@@ -1173,6 +1195,7 @@ int parse_rc(int fd,char *message,char *arguments,struct ir_remote **remote,
 	{
 		return(send_error(fd,message,"code missing\n"));
 	}
+	*err = 0;
 	return(1);
 }
 
@@ -1347,8 +1370,11 @@ int list(int fd,char *message,char *arguments)
 {
 	struct ir_remote *remote;
 	struct ir_ncode *code;
+	int err;
 
-	if(parse_rc(fd,message,arguments,&remote,&code,NULL,0)==0) return(0);
+	if(parse_rc(fd, message, arguments,
+		    &remote, &code, NULL, 0, &err) == 0) return 0;
+	if(err) return 1;
 	
 	if(remote==NULL)
 	{
@@ -1495,14 +1521,15 @@ int send_core(int fd,char *message,char *arguments,int once)
 	struct ir_ncode *code;
 	struct itimerval repeat_timer;
 	int reps;
+	int err;
 	
 	if(hw.send_mode==0) return(send_error(fd,message,"hardware does not "
 					      "support sending\n"));
 	
-	if(parse_rc(fd,message,arguments,&remote,&code,
-		    once ? &reps:NULL,2)==0) return(0);
+	if(parse_rc(fd, message, arguments, &remote, &code,
+		    once ? &reps:NULL, 2, &err) == 0) return(0);
+	if(err) return 1;
 	
-	if(remote==NULL || code==NULL) return(1);
 	if(once)
 	{
 		if(repeat_remote!=NULL)
@@ -1540,7 +1567,7 @@ int send_core(int fd,char *message,char *arguments,int once)
 	else
 	{
 		/* you've been warned, now we have a limit */
-		remote->repeat_countdown=REPEAT_MAX;
+		remote->repeat_countdown = repeat_max;
 	}
 	if(remote->repeat_countdown>0 || code->next != NULL)
 	{
@@ -1583,21 +1610,33 @@ int send_stop(int fd,char *message,char *arguments)
 	struct ir_remote *remote;
 	struct ir_ncode *code;
 	struct itimerval repeat_timer;
+	int err;
 	
-	if(parse_rc(fd,message,arguments,&remote,&code,NULL,2)==0) return(0);
+	if(parse_rc(fd, message, arguments,
+		    &remote, &code, NULL, 0, &err) == 0) return 0;
+	if(err) return 1;
 	
-	if(remote==NULL || code==NULL) return(1);
-	if(repeat_remote && repeat_code &&
-	   strcasecmp(remote->name,repeat_remote->name)==0 && 
-	   strcasecmp(code->name,repeat_code->name)==0)
+	if(repeat_remote && repeat_code)
 	{
 		int done;
+		if(remote &&
+		   strcasecmp(remote->name, repeat_remote->name) != 0)
+		{
+			return(send_error(fd, message, "specified remote "
+					  "does not match\n"));
+		}
+		if(code && strcasecmp(code->name, repeat_code->name) != 0)
+		{
+			return(send_error(fd, message, "specified code "
+					  "does not match\n"));
+		}
 
-		done=REPEAT_MAX-remote->repeat_countdown;
-		if(done<remote->min_repeat)
+		done = repeat_max - repeat_remote->repeat_countdown;
+		if(done < repeat_remote->min_repeat)
 		{
 			/* we still have some repeats to do */
-			remote->repeat_countdown=remote->min_repeat-done;
+			repeat_remote->repeat_countdown =
+				repeat_remote->min_repeat - done;
 			return(send_success(fd,message));
 		}
 		repeat_timer.it_value.tv_sec=0;
@@ -1986,7 +2025,7 @@ int waitfordata(long maxusec)
 				tv.tv_sec= maxusec / 1000000;
 				tv.tv_usec=maxusec % 1000000;
 			}
-			if(hw.fd == -1)
+			if(hw.fd == -1 && use_hw())
 			{
 				/* try to reconnect */
 				timerclear(&timeout);
@@ -2011,7 +2050,8 @@ int waitfordata(long maxusec)
 					struct timeval gap;
 					
 					timersub(&release_time, &now, &gap);
-					if(timercmp(&tv, &gap, >))
+					if(!(timerisset(&tv) || reconnect) ||
+					   timercmp(&tv, &gap, >))
 					{
 						tv = gap;
 					}
@@ -2145,7 +2185,7 @@ void loop()
 {
 	char *message;
 	
-	logprintf(LOG_NOTICE,"lircd(%s) ready", hw.name);
+	logprintf(LOG_NOTICE,"lircd(%s) ready, using %s", hw.name, lircdfile);
 	while(1)
 	{
 		(void) waitfordata(0);
@@ -2207,9 +2247,10 @@ int main(int argc,char **argv)
 #                       if defined(__linux__)
 			{"uinput",no_argument,NULL,'u'},
 #                       endif
+			{"repeat-max",required_argument,NULL,'R'},
 			{0, 0, 0, 0}
 		};
-		c = getopt_long(argc,argv,"hvnp:H:d:o:P:l::c:r::a"
+		c = getopt_long(argc,argv,"hvnp:H:d:o:P:l::c:r::aR:"
 #                               if defined(__linux__)
 				"u"
 #                               endif
@@ -2247,6 +2288,7 @@ int main(int argc,char **argv)
 #                       if defined(__linux__)
 			printf("\t -u --uinput\t\tgenerate Linux input events\n");
 #                       endif
+			printf("\t -R --repeat-max=limit\t\tallow at most this many repeats\n");
 			return(EXIT_SUCCESS);
 		case 'v':
 			printf("%s %s\n",progname,VERSION);
@@ -2351,6 +2393,9 @@ int main(int argc,char **argv)
 			useuinput=1;
 			break;
 #               endif
+		case 'R':
+			repeat_max = atoi(optarg);
+			break;
 		default:
 			printf("Usage: %s [options] [config-file]\n",progname);
 			return(EXIT_FAILURE);
