@@ -1,4 +1,4 @@
-/*      $Id: receive.c,v 5.36 2009/04/26 09:20:21 lirc Exp $      */
+/*      $Id: receive.c,v 5.38 2009/07/08 16:05:16 lirc Exp $      */
 
 /****************************************************************************
  ** receive.c ***************************************************************
@@ -970,6 +970,44 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 		}
 		return code;
 	}
+	else if(is_xmp(remote))
+	{
+		lirc_t deltap,deltas,sum;
+		ir_code n;
+		
+		if(bits%4 || done%4)
+		{
+			logprintf(LOG_ERR,"invalid bit number.");
+			return((ir_code) -1);
+		}
+		if(!sync_pending_space(remote)) return 0;
+		for(i=0;i<bits;i+=4)
+		{
+			code<<=4;
+			deltap=get_next_pulse(remote->pzero);
+			deltas=get_next_space(remote->szero+16*remote->sone);
+			if(deltap==0 || deltas==0) 
+			{
+				logprintf(LOG_ERR,"failed on bit %d",
+					  done+i+1);
+				return((ir_code) -1);
+			}
+			sum=deltap+deltas;
+			
+			sum -= remote->pzero + remote->szero;
+			n = (sum + remote->sone/2)/remote->sone;
+			if(n >= 16)
+			{
+				logprintf(LOG_ERR,"failed on bit %d",
+					  done+i+1);
+				return((ir_code) -1);
+			}
+			LOGPRINTF(1, "%d: %lx", i, n);
+			code |= n;
+
+		}
+		return code;
+	}
 	
 	for(i=0;i<bits;i++)
 	{
@@ -1056,7 +1094,7 @@ int receive_decode(struct ir_remote *remote,
 		   int *repeat_flagp,
 		   lirc_t *min_remaining_gapp, lirc_t *max_remaining_gapp)
 {
-	ir_code pre,code,post,code_mask=0,post_mask=0;
+	ir_code pre,code,post;
 	lirc_t sync;
 	int header;
 	struct timeval current;
@@ -1198,13 +1236,13 @@ int receive_decode(struct ir_remote *remote,
 		if(hw.rec_mode==LIRC_MODE_CODE ||
 		   hw.rec_mode==LIRC_MODE_LIRCCODE)
 		{
-			int i;
  			lirc_t sum;
+			ir_code decoded = rec_buffer.decoded;
 
 #                       ifdef LONG_IR_CODE
-			LOGPRINTF(1,"decoded: %llx",rec_buffer.decoded);
+			LOGPRINTF(1,"decoded: %llx", decoded);
 #                       else
-			LOGPRINTF(1,"decoded: %lx",rec_buffer.decoded);
+			LOGPRINTF(1,"decoded: %lx", decoded);
 #                       endif
 			if((hw.rec_mode==LIRC_MODE_CODE &&
 			    hw.code_length<bit_count(remote))
@@ -1215,21 +1253,11 @@ int receive_decode(struct ir_remote *remote,
 				return(0);
 			}
 			
-			for(i=0;i<remote->post_data_bits;i++)
-			{
-				post_mask=(post_mask<<1)+1;
-			}
-			post=rec_buffer.decoded&post_mask;
-			post_mask=0;
-			rec_buffer.decoded=
-			rec_buffer.decoded>>remote->post_data_bits;
-			for(i=0;i<remote->bits;i++)
-			{
-				code_mask=(code_mask<<1)+1;
-			}
-			code=rec_buffer.decoded&code_mask;
-			code_mask=0;
-			pre=rec_buffer.decoded>>remote->bits;
+			post = decoded & gen_mask(remote->post_data_bits);
+			decoded >>= remote->post_data_bits;
+			code = decoded & gen_mask(remote->bits);
+			pre = decoded >> remote->bits;
+			
 			gettimeofday(&current,NULL);
 			sum=remote->phead+remote->shead+
 				lirc_t_max(remote->pone+remote->sone,
