@@ -1,4 +1,4 @@
-/*      $Id: irxevent.c,v 5.13 2005/08/04 19:56:50 lirc Exp $      */
+/*      $Id: irxevent.c,v 5.19 2007/03/30 07:13:47 lirc Exp $      */
 
 /****************************************************************************
  ** irxevent.c **************************************************************
@@ -92,16 +92,20 @@
 
 #include "lirc_client.h"
 
+static int bDaemon = 0;
+static int bInError = 0;
+
 #ifdef DEBUG
-void debugprintf(char *format_str, ...)
+static void debugprintf(char *format_str, ...)
 {
         va_list ap;
         va_start(ap,format_str);
-        vfprintf(stderr,format_str,ap);
+        if(!bDaemon)
+		vfprintf(stderr,format_str,ap);
         va_end(ap);
 }
 #else
-void debugprintf(char *format_str, ...)
+static void debugprintf(char *format_str, ...)
 {
 }
 #endif
@@ -123,18 +127,18 @@ static struct keymodlist_t keymodlist[]=
   {NULL,0},
 };
 
-const char *key_delimiter ="-";
-const char *active_window_name ="CurrentWindow";
-const char *root_window_name ="RootWindow";
+static const char *key_delimiter ="-";
+static const char *active_window_name ="CurrentWindow";
+static const char *root_window_name ="RootWindow";
 
 
-char *progname;
-Display *dpy;
-Window root;
-XEvent xev;
-Window w,subw;
+static const char *progname="irxevent";
+static Display *dpy;
+static Window root;
+static XEvent xev;
+static Window w,subw;
 
-Time fake_timestamp()
+static Time fake_timestamp()
      /*seems that xfree86 computes the timestamps like this     */
      /*strange but it relies on the *1000-32bit-wrap-around     */
      /*if anybody knows exactly how to do it, please contact me */
@@ -149,7 +153,7 @@ Time fake_timestamp()
   return (Time)tint;
 }
 
-Window find_window(Window top,char *name)
+static Window find_window(Window top,char *name)
 {
   char *wname,*iname;
   XClassHint xch;
@@ -208,7 +212,7 @@ Window find_window(Window top,char *name)
   return(top);
 }
 
-Window find_sub_sub_window(Window top,int *x, int *y)
+static Window find_sub_sub_window(Window top,int *x, int *y)
 {
   Window base;
   Window *children,foo,target=0;
@@ -252,7 +256,7 @@ Window find_sub_sub_window(Window top,int *x, int *y)
 
 
 
-Window find_sub_window(Window top,char *name,int *x, int *y)
+static Window find_sub_window(Window top,char *name,int *x, int *y)
 {
   Window base;
   Window *children,foo,target=0;
@@ -295,7 +299,7 @@ Window find_sub_window(Window top,char *name,int *x, int *y)
 }
 
 
-Window find_window_focused(Window top,char *name) 
+static Window find_window_focused(Window top,char *name) 
 {
   int tmp;
   Window w, cur, *children, foo;
@@ -328,7 +332,7 @@ Window find_window_focused(Window top,char *name)
   return(0);
 }
 
-void make_button(int button,int x,int y,XButtonEvent *xev)
+static void make_button(int button,int x,int y,XButtonEvent *xev)
 {
   xev->type = ButtonPress;
   xev->display=dpy;
@@ -344,7 +348,7 @@ void make_button(int button,int x,int y,XButtonEvent *xev)
   return;
 }
 
-void make_key(char *keyname,int x, int y,XKeyEvent *xev)
+static void make_key(char *keyname,int x, int y,XKeyEvent *xev)
 {
   char *part, *part2, *sep_part;
   struct keymodlist_t *kmlptr;
@@ -414,7 +418,7 @@ void make_key(char *keyname,int x, int y,XKeyEvent *xev)
   return ;
 }
 
-void sendfocus(Window w,int in_out)
+static void sendfocus(Window w,int in_out)
 {
   XFocusChangeEvent focev;
 
@@ -429,7 +433,7 @@ void sendfocus(Window w,int in_out)
   return;
 }
 
-void sendpointer_enter_or_leave(Window w,int in_out)
+static void sendpointer_enter_or_leave(Window w,int in_out)
 {
   XCrossingEvent crossev;
   crossev.type=in_out;
@@ -452,7 +456,7 @@ void sendpointer_enter_or_leave(Window w,int in_out)
   return;
 }
 
-void sendkey(char *keyname,int x,int y,Window w,Window s)
+static void sendkey(char *keyname,int x,int y,Window w,Window s)
 {
   make_key(keyname ,x,y,(XKeyEvent*)&xev);
   xev.xkey.window=w;
@@ -470,7 +474,7 @@ void sendkey(char *keyname,int x,int y,Window w,Window s)
   return;
 }
 
-void sendbutton(int button, int x, int y, Window w,Window s)
+static void sendbutton(int button, int x, int y, Window w,Window s)
 {
   make_button(button,x,y,(XButtonEvent*)&xev);
   xev.xbutton.window=w;
@@ -492,6 +496,17 @@ void sendbutton(int button, int x, int y, Window w,Window s)
   return;
 }
 
+int errorHandler(Display* di, XErrorEvent* ev)
+{
+  char buff[512]; buff[0] = 0;
+  if(bInError || ev==NULL || di==NULL) return 1;      // only 1 msg per key
+  XGetErrorText(di, ev->error_code, buff, sizeof(buff)-1);
+  if(buff[0]) {
+     if(!bDaemon) fprintf(stderr, "X11 error: %s\n", buff);
+     bInError = 1;
+  }
+  return 1;
+}
 
 int check(char *s)
 {
@@ -528,7 +543,8 @@ int check(char *s)
 
 static struct option long_options[] =
 {
-	{"help", no_argument, NULL, 'h'},
+	{"daemon",  no_argument, NULL, 'd'},
+	{"help",    no_argument, NULL, 'h'},
 	{"version", no_argument, NULL, 'V'},
 	{0, 0, 0, 0}
 };
@@ -543,14 +559,15 @@ int main(int argc, char *argv[])
   int c;
   int WindowID;
 
-  progname=argv[0];
-
-  while ((c = getopt_long(argc, argv, "hV", long_options, NULL)) != EOF) {
+  while ((c = getopt_long(argc, argv, "dhV", long_options, NULL)) != EOF) {
     switch (c) {
+    case 'd':
+      bDaemon = 1; continue;
     case 'h':
-      printf("Usage: %s [config file]\n", argv[0]);
-      printf("\t -h --help \t\tdisplay usage summary\n");
-      printf("\t -V --version \t\tdisplay version\n");
+      printf("Usage: %s [option]... [config file]\n"
+             "       -d --daemon     fork and run in background\n"
+             "       -h --help       display usage summary\n"
+             "       -V --version    display version\n", progname);
       return(EXIT_SUCCESS);
     case 'V':
       printf("%s %s\n", progname, VERSION);
@@ -577,6 +594,9 @@ int main(int argc, char *argv[])
   }
   root=RootWindow(dpy,DefaultScreen(dpy));
 
+  // windows may get closed at wrong time. Override default error handler...
+  XSetErrorHandler(errorHandler);
+  
   if(lirc_init("irxevent",1)==-1) exit(EXIT_FAILURE);
 
   if(lirc_readconfig(config_file,&config,check)==0)
@@ -584,13 +604,21 @@ int main(int argc, char *argv[])
       char *ir;
       char *c;
       int ret;
+
+      if(bDaemon) {
+	if(daemon(1, 0) < 0) {
+	  perror("Failed to run as daemon");
+	  exit(EXIT_FAILURE);
+        }
+      }
       
       while(lirc_nextcode(&ir)==0)
 	{
 	  if(ir==NULL) continue;
 	  while((ret=lirc_code2char(config,ir,&c))==0 && c!=NULL)
 	    {
-	      debugprintf("Recieved code: %s Sending event: \n",ir);
+	      debugprintf("Received code: %s Sending event: \n",ir);
+	      bInError = 0;   // reset error state, want to see error msg
 	      
 	      *windowname=0;
 	      if(2==sscanf(c,"Key %s Focus WindowID %i",keyname,&WindowID) ||
@@ -630,6 +658,11 @@ int main(int argc, char *argv[])
 			4==sscanf(c,"xy_Key %d %d %s %s\n",&pointer_x,&pointer_y,keyname,windowname)) {
 		debugprintf("name: %s\n",windowname);
 		WindowID=find_window(root,windowname);
+		if(WindowID == 0)
+		{
+		  debugprintf("target window '%s' not found\n", windowname);
+		  continue;
+		}
 	      }
 	      
 	      switch(c[0])
